@@ -3,62 +3,16 @@ import os
 import re
 import subprocess
 import logging
-
+from chromogenic.settings import chromo_settings
 logger = logging.getLogger(__name__)
 
 ##
 # Tools
 ##
 
-def atmo_required_files(mounted_path):
-    #Add the atmosphere pub-key to every instance, because SSH key injection can fail..
-    inject_atmo_key(mounted_path)
-    inject_denyhosts_file(mounted_path)
-
-def inject_atmo_key(mounted_path):
-    append_line_list = [
-        ("ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDVVkgGS8QwHet+aF401l6MLD206yfE76Pe8UAbWhKdE1155IHyDumS5226cTf+5/1zqyzlGwvHJMhzJEztImJghXAWMw7AOzDUYmIpGGhnvmVE1mJN6Iy3aRDyJOcPOqd1ZGbywzzQioiYjoxKa/HT5QN5F/4Mdsqn3mgFdWgXxmY7X3fZGphk5vOK/8J8tSpy4dLIBI+WRrN4ZR7IOrvzkZght/YjtvgPhJqZzgEzcTP4BMpUNWlOFL95Usk3lzqJTBDzlM71ivaHQ3OqxrjpThMSGoQhedupsx8FrmBvOo1OxjfIj0/hIEtjH9FE2lc5GZBy7B1EuqXApR7Vopa3 atmo@iplantcollaborative.org\n","root/.ssh/authorized_keys"),
-    ]
-    #Ensure SSH Directory exists
-    ssh_dir = os.path.join(mounted_path,"root/.ssh/")
-    if not os.path.isdir(ssh_dir):
-        os.makedirs(ssh_dir)
-    #Attempt to append.
-    append_line_in_files(append_line_list, mounted_path)
-
-def inject_denyhosts_file(mounted_path, denyhosts_file="var/lib/denyhosts/allowed-hosts"):
-    """
-    IF image uses 'denyhosts' add these allowed hosts
-    """
-    denyhosts_folder = os.path.dirname(denyhosts_file)
-    #Skip denyhost injection if denyhost is not installed!
-    if not os.path.isdir(denyhosts_folder):
-        return
-    test_denyhosts_file = os.path.join(mounted_path, denyhosts_file)
-    if not os.path.exists(test_denyhosts_file):
-        return
-
-    #Create some whitelists for denyhosts:
-    ALLOWED_HOST_LIST = [
-        ("128.196.172.*", denyhosts_file),
-        ("128.196.142.*", denyhosts_file),
-        ("128.196.64.*", denyhosts_file),
-        ("128.196.65.*", denyhosts_file),
-        ("128.196.38.*", denyhosts_file),
-        ("150.135.78.*", denyhosts_file),
-        ("150.135.93.*", denyhosts_file),
-    ]
-    text_to_write = "\n".join([rule[0] for rule in ALLOWED_HOST_LIST])
-    if not create_file(
-            denyhosts_file, mounted_path, text_to_write):
-        #Create_file failed (File exists -- Append the list.)
-        append_line_in_files(ALLOWED_HOST_LIST, mounted_path)
-    hosts_allow_list = [("ALL: %s" % rule.replace("*",""), "etc/hosts.allow")
-                        for rule,_ in ALLOWED_HOST_LIST]
-    append_line_in_files(hosts_allow_list, mounted_path)
 
 def run_command(commandList, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                stdin=None, dry_run=False, shell=False):
+                stdin=None, dry_run=False, shell=False, check_return=False):
     """
     NOTE: Use this to run ANY system command, because its wrapped around a loggger
     Using Popen, run any command at the system level and record the output and error streams
@@ -70,7 +24,7 @@ def run_command(commandList, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
         #Bail before making the call
         logger.debug("Mock Command: %s" % cmd_str)
         return ('','')
-    logger.info("Run Command: %s" % cmd_str)
+    #Execution
     try:
         if stdin:
             proc = subprocess.Popen(commandList, stdout=stdout, stderr=stderr,
@@ -79,20 +33,35 @@ def run_command(commandList, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
             proc = subprocess.Popen(commandList, stdout=stdout, stderr=stderr,
                     shell=shell)
         out,err = proc.communicate(input=stdin)
+        return_code = proc.returncode
+        logger.info("Completed Command with exit code %s: %s" % (return_code, cmd_str))
+        if check_return and return_code != 0:
+                raise Exception("Command returned a non-zero exit code (%s) : %s " % (return_code, cmd_str))
+    except Exception, e:
+        logger.exception("Failed command: %s" % cmd_str)
+        logger.exception(e)
+        raise
+
+    #logging - NEVER let logging the commands be the reason
+    # run command fails.
+    try:
+        if stdin:
+            logger.debug("%s STDIN: %s" % (cmd_str, stdin))
+        logger.debug("%s STDOUT: %s" % (cmd_str, out))
+        logger.debug("%s STDERR: %s" % (cmd_str, err))
     except Exception, e:
         logger.exception(e)
-    if stdin:
-        logger.debug("%s STDIN: %s" % (cmd_str, stdin))
-    logger.debug("%s STDOUT: %s" % (cmd_str, out))
-    logger.debug("%s STDERR: %s" % (cmd_str, err))
+
     return (out,err)
 
 def overwrite_file(filepath, dry_run=False):
+    if '*' in filepath:
+        return wildcard_overwrite_file(filepath, dry_run=dry_run)
     if not os.path.exists(filepath):
-        logger.info("Cannot copy /dev/null to non-existent file: %s" %
+        logger.debug("Cannot `truncate -s0` to non-existent file: %s" %
                 filepath)
         return
-    cmd_list = ['/bin/cp', '-f', '/dev/null', '%s' % filepath]
+    cmd_list = ['/usr/bin/truncate', '-s0', '%s' % filepath]
     run_command(cmd_list, dry_run=dry_run)
 
 
@@ -103,17 +72,32 @@ def create_file(filepath, mount_point, text_to_write, dry_run=False):
         logger.warn("Cannot create file %s, the file already exists."
                     % create_file_path)
         return False
-    with open(create_file_path, 'w') as the_file:
-        #Write the text, end with empty line
-        the_file.write('%s\n' % text_to_write)
+    write_file(create_file_path, text_to_write)
     return True
 
 
+def write_file(filepath, text_to_write):
+    with open(filepath, 'w') as the_file:
+        #Write the text, end with empty line
+        the_file.write('%s\n' % text_to_write)
+    logger.info("%s written to file: %s" % (text_to_write, filepath))
+
+def wildcard_overwrite_file(wildcard_path, dry_run=False):
+    """
+    Expand the wildcard to match all files, delete each one.
+    """
+    logger.info("Wildcard remove: %s" % wildcard_path)
+    glob_list = glob.glob(wildcard_path)
+    if glob_list:
+        for filepath in glob_list:
+            cmd_list = ['/usr/bin/truncate', '-s0', '%s' % filepath]
+            run_command(cmd_list, dry_run=dry_run)
+
 def wildcard_remove(wildcard_path, dry_run=False):
     """
     Expand the wildcard to match all files, delete each one.
     """
-    logger.debug("Wildcard remove: %s" % wildcard_path)
+    logger.info("Wildcard remove: %s" % wildcard_path)
     glob_list = glob.glob(wildcard_path)
     if glob_list:
         for filename in glob_list:
@@ -126,7 +110,7 @@ BE VERY CAREFUL USING THESE -- YOU HAVE BEEN WARNED!
 """
 def sed_delete_multi(from_here,to_here,filepath, dry_run=False):
     if not os.path.exists(filepath):
-        logger.info("File not found: %s Cannot delete lines" % filepath)
+        logger.warn("File not found: %s Cannot delete lines" % filepath)
         return
     cmd_list = ["/bin/sed", "-i", "/%s/,/%s/d" % (from_here, to_here),
                 filepath]
@@ -134,21 +118,21 @@ def sed_delete_multi(from_here,to_here,filepath, dry_run=False):
 
 def sed_replace(find,replace,filepath, dry_run=False):
     if not os.path.exists(filepath):
-        logger.info("File not found: %s Cannot replace lines" % filepath)
+        logger.warn("File not found: %s Cannot replace lines" % filepath)
         return
     cmd_list = ["/bin/sed", "-i", "s/%s/%s/" % (find,replace), filepath]
     run_command(cmd_list, dry_run=dry_run)
 
 def sed_delete_one(remove_string, filepath, dry_run=False):
     if not os.path.exists(filepath):
-        logger.info("File not found: %s Cannot delete lines" % filepath)
+        logger.warn("File not found: %s Cannot delete lines" % filepath)
         return
     cmd_list = ["/bin/sed", "-i", "/%s/d" % remove_string, filepath]
     run_command(cmd_list, dry_run=dry_run)
 
 def sed_append(append_string, filepath, dry_run=False):
     if not os.path.exists(filepath):
-        logger.info("File not found: %s Cannot append lines" % filepath)
+        logger.warn("File not found: %s Cannot append lines" % filepath)
         return
     if _line_exists_in_file(append_string, filepath):
         return
@@ -157,7 +141,7 @@ def sed_append(append_string, filepath, dry_run=False):
 
 def sed_prepend(prepend_string, filepath, dry_run=False):
     if not os.path.exists(filepath):
-        logger.info("File not found: %s Cannot prepend lines" % filepath)
+        logger.warn("File not found: %s Cannot prepend lines" % filepath)
         return
     if _line_exists_in_file(prepend_string, filepath):
         return
@@ -294,7 +278,7 @@ def check_root():
 def mount_image(image_path, mount_point):
     if not check_root():
         raise Exception("Only the root user can mount an image.")
-    if not check_dir(mount_point):
+    if not os.path.isdir(mount_point):
         os.makedirs(mount_point)
     return _detect_and_mount_image(image_path, mount_point)
 
@@ -321,29 +305,21 @@ def create_empty_image(new_image_path, image_type='raw',
     return new_image_path
 
 
-##
-# Validation
-##
-
-def check_file(file_path):
-    return os.path.isfile(file_path)
-
-
-def check_dir(dir_path):
-    return os.path.isdir(dir_path)
-
-
 ##
 # Private Methods
 ##
 
 def append_line_in_files(append_files, mount_point, dry_run=False):
+    if not append_files:
+        return
     for (append_line, append_to) in append_files:
         append_to = _check_mount_path(append_to)
         mounted_filepath = os.path.join(mount_point, append_to)
         sed_append(append_line, mounted_filepath, dry_run=dry_run)
 
 def prepend_line_in_files(prepend_files, mount_point, dry_run=False):
+    if not prepend_files:
+        return
     for (prepend_line, prepend_to) in prepend_files:
         prepend_to = _check_mount_path(prepend_to)
         mounted_filepath = os.path.join(mount_point, prepend_to)
@@ -363,7 +339,7 @@ def remove_files(rm_files, mount_point, dry_run=False):
 
 def overwrite_files(overwrite_files, mount_point, dry_run=False):
     """
-    #Copy /dev/null to clear sensitive logging data
+    #Truncate files to clear sensitive logging data
     """
     for overwrite_path in overwrite_files:
         overwrite_path = _check_mount_path(overwrite_path)
@@ -392,6 +368,24 @@ def replace_line_in_files(replace_line_files, mount_point, dry_run=False):
                     dry_run=dry_run)
 
 
+def execute_chroot_commands(subprocess_commands, mounted_path, dry_run=False):
+    """
+    Execute the following command(s) inside a chroot jail
+    """
+    # If empty -- do nothing.
+    if not subprocess_commands:
+        return
+    try:
+        prepare_chroot_env(mounted_path)
+        for cmd_list in subprocess_commands:
+            if 'chroot' not in cmd_list[0]:
+                cmd_list = ["/usr/sbin/chroot", mounted_path] + cmd_list
+            logger.info(cmd_list)
+            run_command(cmd_list, dry_run=dry_run)
+    finally:
+        remove_chroot_env(mounted_path)
+
+
 def remove_multiline_in_files(multiline_delete_files, mount_point, dry_run=False):
     """
     #Remove EVERYTHING between these lines..
@@ -492,40 +486,6 @@ def _detect_and_mount_image(image_path, mount_point):
     raise Exception("The type of image "
             "could not be determined based on the extension:%s" % file_ext)
 
-def check_mounted(mount_point):
-    dev_location = None
-    #Drop trailing slash to match 'mount' syntax
-    if mount_point and mount_point.endswith('/'):
-        mount_point = mount_point[:-1]
-    #Run mount and scan the output for 'mount_point'
-    stdout, stderr = run_command(['mount'])
-    regex = re.compile("(?P<device>[\w/]+) on (?P<location>.*) type")
-    for line in stdout.split('\n'):
-        res = regex.search(line)
-        if not res:
-            continue
-        search_dict = res.groupdict()
-        mount_location = search_dict['location']
-        if mount_point == mount_location:
-            dev_location = search_dict['device']
-
-    return dev_location
-
-def unmount_image(image_path, mount_point):
-    device = check_mounted(mount_point)
-    if not device:
-        return ('', '%s is not mounted' % image_path)
-    # Rely on file, its smarter than a name.
-    file_type = _get_type_by_metadata(image_path)
-
-    if 'qcow' in file_type:
-        return unmount_qcow(device)
-    elif file_type in ['raw','img']:
-        return unmount_raw(device)
-    else:
-        raise Exception("Encountered an unknown image type -- Extension : %s"
-                        % file_ext)
-
 def unmount_raw(block_device):
     #Remove net block device
     out, err = run_command(['umount', block_device])
@@ -546,9 +506,9 @@ def remove_chroot_env(mount_point):
     sys_dir = os.path.join(mount_point,'sys/')
     dev_dir = os.path.join(mount_point,'dev/')
     etc_resolv_file = os.path.join(mount_point,'etc/resolv.conf')
-    run_command(['umount', proc_dir])
-    run_command(['umount', sys_dir])
-    run_command(['umount', dev_dir])
+    run_command(['umount', '-lf', proc_dir], check_return=True)
+    run_command(['umount', '-lf', sys_dir], check_return=True)
+    run_command(['umount', '-lf', dev_dir], check_return=True)
     run_command(['umount', etc_resolv_file])
 
 
@@ -587,6 +547,7 @@ def fsck_qcow(image_path):
     nbd_dev = _get_next_nbd()
     try:
         run_command(['qemu-nbd', '-c', nbd_dev, image_path])
+        run_command(['partprobe', '-s', nbd_dev])
         run_command(['fsck', '-y', nbd_dev])
     finally:
         run_command(['qemu-nbd', '-d', nbd_dev])
@@ -596,12 +557,16 @@ def _get_parted_fs_type(partition_path):
     out, err = run_command(['parted', '-sm', partition_path, 'print'])
     if not out or err:
         return None
+    elif 'unrecognised disk label' in out:
+        return None
     elif 'Input/output error' in out:
         return None
     elif 'xfs' in out:
         return 'xfs'
     elif 'ext3' in out:
         return 'ext3'
+    elif 'ext4' in out:
+        return 'ext4'
     else:
         full_out, _ = run_command(['parted', '-sm', partition_path, 'print'])
         raise Exception("Received 'parted output' of %s -"
@@ -618,29 +583,46 @@ def mount_qcow(image_path, mount_point):
     nbd_dev = _get_next_nbd()
     #Mount disk to /dev/nbd*
     run_command(['qemu-nbd', '-c', nbd_dev, image_path])
+    run_command(['partprobe', '-s', nbd_dev])
     #Check if filesystem has multiple partitions
     try:
         partition = _fdisk_get_partition(nbd_dev)
         mount_from = partition.get('image_name',nbd_dev)
+        offset = int(partition.get('start',0)) *512
         fs_type = _get_parted_fs_type(mount_from)
-    except Exception:
+    except Exception as e:
+        logger.exception(e)
         mount_from = nbd_dev
+        offset = 0
         fs_type = None
     if fs_type == 'xfs':
         _init_xfs(mount_from)
-    try:
-        out, err = run_command(['mount', mount_from, mount_point])
-        if err:
-            raise Exception("Failed to mount QCOW. STDERR: %s" % err)
-        # The qcow image has been mounted
+    mount_success = attempt_mount(mount_from, mount_point)
+    if not mount_success:
+        mount_success = attempt_mount(nbd_dev, mount_point, "offset=%s,nouuid" % offset)
+    if mount_success:
         return True, nbd_dev
-    except Exception:
-        logger.exception('Could not mount QCOW image:%s to device:%s'
+    else:
+        logger.error('Could not mount QCOW image:%s to device:%s'
                          % (image_path, nbd_dev))
-        # Run only on exception.. We want to keep the image mounted!
+        # Run only on complete mount failure.. We want to keep the image mounted!
         run_command(['qemu-nbd', '-d', nbd_dev])
         return False, None
 
+def attempt_mount(mount_from, mount_point, mount_options=None):
+    if mount_options:
+        mount_cmd_list = ['mount', "-o %s" % mount_options, mount_from, mount_point]
+    else:
+        mount_cmd_list = ['mount', mount_from, mount_point]
+    try:
+        out, err = run_command(mount_cmd_list)
+        if err:
+            raise Exception("Failed to mount. STDERR: %s" % err)
+        return True
+    except:
+        logger.exception('Could not mount file:%s to device:%s using options: %s'
+                         % (mount_from, mount_point, mount_options))
+
 
 def fdisk_image(image_path):
     out, err = run_command(['fdisk','-l',image_path])
@@ -712,32 +694,6 @@ def mount_raw_with_offsets(image_path, mount_point):
                 partition)
     return out, err
 
-def prepare_losetup(image_path):
-    out, err = run_command(['fdisk','-l',image_path])
-    fdisk_stats = _parse_fdisk_stats(out)
-    partition = _select_partition(fdisk_stats['devices'])
-    #This is a 'known constant'.. It should never change..
-    #4096 = Default block size for ext2/ext3
-    BLOCK_SIZE = 4096
-
-    #First mount the loopback device
-    loop_offset = part['start'] * disk['logical_sector_size']
-    (loop_str, _) = run_command(['losetup', '-fv', '-o', '%s' % loop_offset,
-        image_path])
-
-    #The last word of the output is the device
-    loop_dev = _losetup_extract_device(loop_str)
-    return loop_dev
-
-def _mount_lvm(image_path, mount_point):
-    """
-    LVM's are one of the more difficult problems..
-    We will save this until it becomes necessary.. And it will.
-    """
-    #vgscan
-    #...
-    pass
-
 
 def _select_partition(partitions):
     """
@@ -851,15 +807,3 @@ def build_imaging_dirs(download_dir, full_image=False):
     if full_image:
         return (kernel_dir, ramdisk_dir, mount_point)
     return mount_point
-
-def qemu_convert(image_location, dest_location, source_ext=None, output_ext=None):
-    if not source_ext:
-        _, source_ext = os.path.splitext(image_location)
-    if not output_ext:
-        _, output_ext = os.path.splitext(dest_location)
-    (out, err) = run_command(['qemu-img', 'convert',
-                            "-f", source_ext,
-                            '-O', output_ext,
-                            image_location, dest_location])
-    return out, err
-
