@@ -38,6 +38,7 @@ except ImportError:
     has_euca = False
     pass
 
+from chromogenic.clean  import mount_and_clean
 from chromogenic.common import run_command, wildcard_remove
 from chromogenic.common import mount_image, get_latest_ramdisk,\
                                _copy_kernel, _copy_ramdisk
@@ -117,15 +118,11 @@ class ImageManager(BaseDriver):
         download_location = self.download_instance(**download_args)
         logger.debug("Instance downloaded to %s" % download_location)
         download_dir = download_args['download_dir']
-        mount_point = self.build_imaging_dirs(download_dir)
 
-        #Step 2. Mount and Clean image 
+        #Step 2. Mount and Clean image
         if kwargs.get('clean_image',True):
             logger.debug("Cleaning image file: %s" % download_location)
-            self.mount_and_clean(
-                    download_location,
-                    mount_point,
-                    **kwargs)
+            mount_and_clean(download_location, **kwargs)
 
         ##Step 3. Upload image
         upload_kwargs = self.parse_upload_args(
@@ -146,8 +143,8 @@ class ImageManager(BaseDriver):
         download_args = {}
         #REQUIRED kwargs:
         #  node_scp_info - Dictionary for accessing the node controller,
-        #  should contain: 
-        #    * hostname, 
+        #  should contain:
+        #    * hostname,
         #    * port,
         #    * username(if not root),
         #    * a private ssh_key that allows access to the box. (if necessary)
@@ -375,7 +372,7 @@ class ImageManager(BaseDriver):
                     image_name, new_image_path,
                     kernel_id, ramdisk_id,
                     download_dir, None, bucket_name,
-                    public, private_users) 
+                    public, private_users)
         finally:
             os.rename(new_image_path,image_path)
 
@@ -425,7 +422,7 @@ class ImageManager(BaseDriver):
 
         run_command(["umount", mount_point])
 
-        #Your new image is ready for upload to OpenStack 
+        #Your new image is ready for upload to OpenStack
         return (image_path, local_kernel_path, local_ramdisk_path)
 
     # __init__ privates
@@ -584,7 +581,7 @@ class ImageManager(BaseDriver):
         logger.debug("Deleted bucket %s" % bucket_name)
         return True
 
-        
+
 
     def _upload_kernel(self, image_path, bucket_name, download_dir='/tmp'):
         return self._upload_and_register(image_path, bucket_name,
@@ -599,7 +596,7 @@ class ImageManager(BaseDriver):
         bucket_name = bucket_name.lower()
         logger.debug('Bundling image %s to dir:%s'
                      % (image_path, download_dir))
-        manifest_loc = self._bundle_image(image_path, download_dir, 
+        manifest_loc = self._bundle_image(image_path, download_dir,
                                           kernel, ramdisk,
                                           ancestor_ami_ids=ancestor_ami_ids)
         logger.debug(manifest_loc)
@@ -609,7 +606,7 @@ class ImageManager(BaseDriver):
                     % new_image_id)
         return new_image_id
 
-    def scp_remote_file(self, remote_path=None, local_path=None, 
+    def scp_remote_file(self, remote_path=None, local_path=None,
                         user='root', hostname='localhost', port=22,
                         check_key=False, private_key=None):
         """
@@ -815,7 +812,6 @@ class ImageManager(BaseDriver):
         try:
             self.euca.validate_file(manifest_path)
         except FileValidationError:
-            print 'Invalid manifest'
             logger.error("Invalid manifest file provided. Check path")
             raise
 
@@ -1047,14 +1043,14 @@ These functions belong to euca-upload-bundle in euca2ools 1.3.1
 
 
 def _create_bucket(connection, bucket, canned_acl=None):
-    print 'Creating bucket:', bucket
+    logger.info( 'Creating bucket:%s' % bucket)
     return connection.create_bucket(bucket, policy=canned_acl)
 
 
 def _ensure_bucket(connection, bucket, canned_acl=None):
     bucket_instance = None
     try:
-        print 'Checking bucket:', bucket
+        logger.info('Checking bucket: %s' % bucket)
         bucket_instance = connection.get_bucket(bucket)
     except S3ResponseError, s3error:
         s3error_string = '%s' % (s3error)
@@ -1063,13 +1059,13 @@ def _ensure_bucket(connection, bucket, canned_acl=None):
                 bucket_instance = _create_bucket(
                     connection, bucket, canned_acl)
             except S3CreateError:
-                print 'Unable to create bucket %s' % (bucket)
+                logger.info('Unable to create bucket %s' % (bucket))
                 sys.exit()
         elif (s3error_string.find("403") >= 0):
-            print "You do not have permission to access bucket:", bucket
+            logger.info("You do not have permission to access bucket: %s" % bucket)
             sys.exit()
         else:
-            print s3error_string
+            logger.warn(s3error_string)
             sys.exit()
     return bucket_instance
 
@@ -1080,7 +1076,6 @@ def _get_relative_filename(filename):
 
 
 def _upload_manifest(bucket_instance, manifest_filename, canned_acl=None):
-    print 'Uploading manifest file'
     k = Key(bucket_instance)
     k.key = _get_relative_filename(manifest_filename)
     manifest_file = open(manifest_filename, "rb")
@@ -1089,9 +1084,9 @@ def _upload_manifest(bucket_instance, manifest_filename, canned_acl=None):
     except S3ResponseError, s3error:
         s3error_string = '%s' % (s3error)
         if (s3error_string.find("403") >= 0):
-            print "Permission denied while writing:", k.key
+            logger.error("Permission denied while writing: %s" % k.key)
         else:
-            print s3error_string
+            logger.error("S3 error: %s" % s3error_string)
         sys.exit()
 
 
@@ -1106,7 +1101,7 @@ def _upload_parts(bucket_instance, directory, parts,
         if part == part_to_start_from:
             okay_to_upload = True
         if okay_to_upload:
-            print 'Uploading part:', part
+            logger.info('Uploading part: %s' % part)
             k = Key(bucket_instance)
             k.key = part
             part_file = open(os.path.join(directory, part), "rb")
@@ -1115,9 +1110,7 @@ def _upload_parts(bucket_instance, directory, parts,
             except S3ResponseError, s3error:
                 s3error_string = '%s' % (s3error)
                 if (s3error_string.find("403") >= 0):
-                    print "Permission denied while writing:", k.key
+                    logger.info("Permission denied while writing: %s" % k.key)
                 else:
-                    print s3error_string
+                    logger.info(s3error_string)
                 sys.exit()
-
-
